@@ -19,10 +19,9 @@ namespace Tomboy
                 Gtk.ScrolledWindow matches_window;
                 Gtk.VBox content_vbox;
                 Gtk.TreeViewColumn matches_column;
-
-		        Gtk.TreeView tags_tree;
-		        Gtk.TreeModel tags_store;
-		
+				
+				Notebooks.NotebooksTreeView notebooksTree;
+				
                 // Use the following like a Set
 		        Dictionary<Tag, Tag> selected_tags;
 
@@ -47,12 +46,16 @@ namespace Tomboy
                 };
 
                 static Gdk.Pixbuf note_icon;
+                static Gdk.Pixbuf all_notes_icon;
+                static Gdk.Pixbuf notebook_icon;
                 static ArrayList previous_searches;
                 static NoteRecentChanges instance;
 
                 static NoteRecentChanges ()
                 {
                         note_icon = GuiUtils.GetIcon ("tomboy-note", 22);
+                        all_notes_icon = GuiUtils.GetIcon ("tomboy-all-notes", 22);
+                        notebook_icon = GuiUtils.GetIcon ("tomboy-notebook", 22);
                 }
 
                 public static NoteRecentChanges GetInstance (NoteManager manager)
@@ -120,29 +123,11 @@ namespace Tomboy
                         hbox.PackStart (image, false, false, 4);
                         hbox.PackStart (table, true, true, 0);
                         hbox.ShowAll ();
-
-					   tags_tree = MakeTagsTree ();
-					   tags_store = TagManager.Tags;
-
-								
-			           TagManager.TagRemoved += OnTagRemoved;
-								
-					   tags_tree.Model = tags_store;
-					   tags_tree.Show ();
-
-					   Gtk.ScrolledWindow tags_sw = new Gtk.ScrolledWindow ();
-					   tags_sw.ShadowType = Gtk.ShadowType.In;
-//
-//                         Reign in the window size if there are tags with long
-//                         names, or a lot of tags...
-
-					   Gtk.Requisition tags_tree_req = tags_tree.SizeRequest ();
-					   if (tags_tree_req.Width > 150)
-					    tags_sw.WidthRequest = 150;
-
-					   tags_sw.Add (tags_tree);
-					   tags_sw.Show ();
-
+						
+						// Notebooks Pane
+						Gtk.Widget notebooksPane = MakeNotebooksPane ();
+						notebooksPane.Show ();
+                        
                         MakeRecentTree ();
                         tree.Show ();
 
@@ -181,7 +166,7 @@ namespace Tomboy
 
 					   Gtk.HPaned hpaned = new Gtk.HPaned ();
 					   hpaned.Position = 150;
-					   hpaned.Add1 (tags_sw);
+						hpaned.Add1 (notebooksPane);
 					   hpaned.Add2 (matches_window);
 					   hpaned.Show ();
 
@@ -207,6 +192,12 @@ namespace Tomboy
                         this.Add (content_vbox);
                         this.DeleteEvent += OnDelete;
                         this.KeyPressEvent += OnKeyPressed; // For Escape
+                        
+                        // Watch when notes are added to notebooks so the search
+                        // results will be updated immediately instead of waiting
+                        // until the note's QueueSave () kicks in.
+                        Notebooks.NotebookManager.NoteAddedToNotebook += OnNoteAddedToNotebook;
+                        Notebooks.NotebookManager.NoteRemovedFromNotebook += OnNoteRemovedFromNotebook;
                 }
 
                 Gtk.MenuBar CreateMenuBar ()
@@ -217,6 +208,9 @@ namespace Tomboy
 
                         am ["OpenNoteAction"].Activated += OnOpenNote;
                         am ["DeleteNoteAction"].Activated += OnDeleteNote;
+                        am ["OpenNotebookTemplateNoteAction"].Activated += OnOpenNotebookTemplateNote;
+			am ["NewNotebookAction"].Activated += OnNewNotebook;
+                        am ["DeleteNotebookAction"].Activated += OnDeleteNotebook;
                         am ["CloseWindowAction"].Activated += OnCloseWindow;
                         if (Tomboy.TrayIconShowing == false)
                                 am ["CloseWindowAction"].Visible = false;
@@ -234,38 +228,47 @@ namespace Tomboy
                         return menubar;
                 }
 
-                
-                  Gtk.TreeView MakeTagsTree ()
-                  {
-                   Gtk.TreeView t;
-
-                   t = new Gtk.TreeView ();
-                   t.HeadersVisible = true;
-                   t.RulesHint = true;
-
-                   Gtk.CellRenderer renderer;
-
-                   Gtk.TreeViewColumn tags_column = new Gtk.TreeViewColumn ();
-                   tags_column.Title = Catalog.GetString ("Tags");
-                   tags_column.Sizing = Gtk.TreeViewColumnSizing.Autosize;
-                   tags_column.Resizable = false;
-
-                   renderer = new Gtk.CellRendererToggle ();
-                   (renderer as Gtk.CellRendererToggle).Toggled += OnTagToggled;
-                   tags_column.PackStart (renderer, false);
-                   tags_column.SetCellDataFunc (renderer,
-                     new Gtk.TreeCellDataFunc (TagsToggleCellDataFunc));
-
-                   renderer = new Gtk.CellRendererText ();
-                   tags_column.PackStart (renderer, true);
-                   tags_column.SetCellDataFunc (renderer,
-                     new Gtk.TreeCellDataFunc (TagsNameCellDataFunc));
-
-                   t.AppendColumn (tags_column);
-
-                   return t;
-                  }
-                
+		Gtk.Widget MakeNotebooksPane ()
+		{
+			notebooksTree = new Notebooks.NotebooksTreeView (Notebooks.NotebookManager.NotebooksWithAllNotesItem);
+			notebooksTree.Selection.Mode = Gtk.SelectionMode.Single;
+			notebooksTree.HeadersVisible = true;
+			notebooksTree.RulesHint = false;
+			
+			Gtk.CellRenderer renderer;
+			
+			Gtk.TreeViewColumn column = new Gtk.TreeViewColumn ();
+			column.Title = Catalog.GetString ("Notebooks");
+			column.Sizing = Gtk.TreeViewColumnSizing.Autosize;
+			column.Resizable = false;
+			
+			renderer = new Gtk.CellRendererPixbuf ();
+			column.PackStart (renderer, false);
+			column.SetCellDataFunc (renderer,
+				new Gtk.TreeCellDataFunc (NotebookPixbufCellDataFunc));
+			
+			renderer = new Gtk.CellRendererText ();
+			column.PackStart (renderer, true);
+			column.SetCellDataFunc (renderer,
+				new Gtk.TreeCellDataFunc (NotebookTextCellDataFunc));
+			
+			notebooksTree.AppendColumn (column);
+			
+			notebooksTree.RowActivated += OnNotebookRowActivated;
+			notebooksTree.Selection.Changed += OnNotebookSelectionChanged;
+			notebooksTree.ButtonPressEvent += OnNotebooksTreeButtonPressed;
+			notebooksTree.KeyPressEvent += OnNotebooksKeyPressed;
+			
+			notebooksTree.Show ();
+			Gtk.ScrolledWindow sw = new Gtk.ScrolledWindow ();
+			sw.HscrollbarPolicy = Gtk.PolicyType.Automatic;
+			sw.VscrollbarPolicy = Gtk.PolicyType.Automatic;
+			sw.ShadowType = Gtk.ShadowType.In;
+			sw.Add (notebooksTree);
+			sw.Show ();
+			
+			return sw;
+		}
 
                 void MakeRecentTree ()
                 {
@@ -373,12 +376,6 @@ namespace Tomboy
 
                         tree.Model = store_sort;
 
-                        note_count.Text = string.Format (
-                                                  Catalog.GetPluralString("Total: {0} note",
-                                                                          "Total: {0} notes",
-                                                                          cnt),
-                                                  cnt);
-
                         PerformSearch ();
 
                         if (sort_column >= 0) {
@@ -421,7 +418,7 @@ namespace Tomboy
 			if (text == null) {
 				current_matches.Clear ();
 				store_filter.Refilter ();
-				UpdateNoteCount (store.IterNChildren (), -1);
+				UpdateTotalNoteCount (store_sort.IterNChildren ());
 				if (tree.IsRealized)
 					tree.ScrollToPoint (0, 0);
 				return;
@@ -430,17 +427,22 @@ namespace Tomboy
 				text = text.ToLower ();
 
 			current_matches.Clear ();
-
+			
+			// Search using the currently selected notebook
+			Notebooks.Notebook selected_notebook = GetSelectedNotebook ();
+			if (selected_notebook is Notebooks.AllNotesNotebook)
+				selected_notebook = null;
+			
 			IDictionary<Note,int> results =
-				search.SearchNotes(text, case_sensitive.Active);
+				search.SearchNotes(text, case_sensitive.Active, selected_notebook);
 			foreach (Note note in results.Keys){
 				current_matches.Add(note.Uri, results[note]);
 			}
 			
-			UpdateNoteCount (store.IterNChildren (), current_matches.Count);
 			AddMatchesColumn ();
 			store_filter.Refilter ();
 			tree.ScrollToPoint (0, 0);
+			UpdateMatchNoteCount (current_matches.Count);
 		}
 
                 void AddMatchesColumn ()
@@ -511,33 +513,42 @@ namespace Tomboy
 
                         crt.Text = match_str;
                 }
-
-                void UpdateNoteCount (int total, int matches)
+                
+                void UpdateTotalNoteCount (int total)
                 {
-                        string cnt = string.Format (
-                                             Catalog.GetPluralString("Total: {0} note",
-                                                                     "Total: {0} notes",
-                                                                     total),
-                                             total);
-                        if (matches >= 0) {
-                                cnt += ", " + string.Format (
-                                               Catalog.GetPluralString ("{0} match",
-                                                                        "{0} matches",
-                                                                        matches),
-                                               matches);
-                        }
-                        note_count.Text = cnt;
+                	string status = string.Format (
+                			Catalog.GetPluralString ("Total: {0} note",
+                									 "Total: {0} notes",
+                									 total),
+                			total);
+                	note_count.Text = status;
                 }
-
+                
+                void UpdateMatchNoteCount (int matches)
+                {
+                	string status = string.Format (
+                			Catalog.GetPluralString ("Matches: {0} note",
+                									 "Matches: {0} notes",
+                									 matches),
+                			matches);
+                	note_count.Text = status;
+                }
+                
                 /// <summary>
                 /// Filter out notes based on the current search string
-                /// and selected tags.
+                /// and selected tags.  Also prevent template notes from
+                /// appearing.
                 /// </summary>
                 bool FilterNotes (Gtk.TreeModel model, Gtk.TreeIter iter)
                 {
                         Note note = model.GetValue (iter, 3 /* note */) as Note;
                         if (note == null)
                                 return false;
+                        
+                        // Don't show the template notes in the list
+                        Tag template_tag = TagManager.GetOrCreateSystemTag (TagManager.TemplateNoteSystemTag);
+                        if (note.ContainsTag (template_tag))
+                        	return false;
 
                         bool passes_search_filter = FilterBySearch (note);
                         if (passes_search_filter == false)
@@ -552,15 +563,11 @@ namespace Tomboy
 		
 		        bool FilterTags (Gtk.TreeModel model, Gtk.TreeIter iter)
                 {
-                        Tag t = model.GetValue (iter, 0 /* note */) as Tag;
-			             if(t.IsProperty || t.IsSystem)
-				             return false;
-			             if(t.Name.StartsWith("system:"))
-				             return false;
-			             if(t.NormalizedName.StartsWith("system:"))
-				             return false;
-			             return true;
-            
+						Tag t = model.GetValue (iter, 0 /* note */) as Tag;
+						if(t.IsProperty || t.IsSystem)
+							return false;
+						
+						return true;
                 }
 
                 // <summary>
@@ -665,17 +672,18 @@ namespace Tomboy
                                 if (selection.CountSelectedRows () == 0)
                                         break;
 
-                                PopupContextMenuAtLocation ((int) args.Event.X,
+		                        Gtk.Menu menu = Tomboy.ActionManager.GetWidget (
+                                            "/MainWindowContextMenu") as Gtk.Menu;
+                                PopupContextMenuAtLocation (menu,
+                                							(int) args.Event.X,
                                                             (int) args.Event.Y);
 
                                 break;
                         }
                 }
 
-                void PopupContextMenuAtLocation (int x, int y)
+                void PopupContextMenuAtLocation (Gtk.Menu menu, int x, int y)
                 {
-                        Gtk.Menu menu = Tomboy.ActionManager.GetWidget (
-                                                "/MainWindowContextMenu") as Gtk.Menu;
                         menu.ShowAll ();
                         Gtk.MenuPositionFunc pos_menu_func = null;
 
@@ -785,6 +793,9 @@ namespace Tomboy
                         manager.NoteAdded -= OnNotesChanged;
                         manager.NoteRenamed -= OnNoteRenamed;
                         manager.NoteSaved -= OnNoteSaved;
+                        
+                        Notebooks.NotebookManager.NoteAddedToNotebook -= OnNoteAddedToNotebook;
+                        Notebooks.NotebookManager.NoteRemovedFromNotebook -= OnNoteRemovedFromNotebook;
 
                         // The following code has to be done for the MenuBar to
                         // appear properly the next time this window is opened.
@@ -793,6 +804,9 @@ namespace Tomboy
                                 ActionManager am = Tomboy.ActionManager;
                                 am ["OpenNoteAction"].Activated -= OnOpenNote;
                                 am ["DeleteNoteAction"].Activated -= OnDeleteNote;
+				am ["NewNotebookAction"].Activated -= OnNewNotebook;
+				am ["DeleteNotebookAction"].Activated -= OnDeleteNotebook;
+				am ["OpenNotebookTemplateNoteAction"].Activated -= OnOpenNotebookTemplateNote;
                                 am ["CloseWindowAction"].Activated -= OnCloseWindow;
                         }
 
@@ -820,18 +834,24 @@ namespace Tomboy
                         case Gdk.Key.Menu:
                                 // Pop up the context menu if a note is selected
                                 Note note = GetSelectedNote ();
-                                if (note != null)
-                                        PopupContextMenuAtLocation (0, 0);
+                                if (note != null) {
+			                        Gtk.Menu menu = Tomboy.ActionManager.GetWidget (
+                                                "/MainWindowContextMenu") as Gtk.Menu;
+                                    PopupContextMenuAtLocation (menu, 0, 0);
+                                }
 
                                 break;
                         }
                 }
 
-                protected override void OnShown ()
-                {
-                        find_combo.Entry.GrabFocus ();
-                        base.OnShown ();
-                }
+		protected override void OnShown ()
+		{
+			// Select "All Notes" in the notebooks list
+			SelectAllNotesNotebook ();
+			
+			find_combo.Entry.GrabFocus ();
+			base.OnShown ();
+		}
 
                 int CompareTitles (Gtk.TreeModel model, Gtk.TreeIter a, Gtk.TreeIter b)
                 {
@@ -984,81 +1004,194 @@ namespace Tomboy
                         find_combo.Entry.GrabFocus ();
                 }
 
-                
-                  // <summary>
-                  // Pay attention to when tags are removed so selected_tags
-                  // remains up-to-date if a selected tag is removed from
-                  // the system.
-                  // </summary>
-                  void OnTagRemoved (string tag_name)
-                  {
-                   if (selected_tags.Count == 0)
-                    return;
+		private void NotebookPixbufCellDataFunc (Gtk.TreeViewColumn treeColumn,
+				Gtk.CellRenderer renderer, Gtk.TreeModel model,
+				Gtk.TreeIter iter)
+		{
+			Notebooks.Notebook notebook = model.GetValue (iter, 0) as Notebooks.Notebook;
+			if (notebook == null)
+				return;
+			
+			Gtk.CellRendererPixbuf crp = renderer as Gtk.CellRendererPixbuf;
+			if (notebook is Notebooks.AllNotesNotebook) {
+				crp.Pixbuf = all_notes_icon;
+			} else {
+				crp.Pixbuf = notebook_icon;
+			}
+		}
+		
+		private void NotebookTextCellDataFunc (Gtk.TreeViewColumn treeColumn,
+				Gtk.CellRenderer renderer, Gtk.TreeModel model,
+				Gtk.TreeIter iter)
+		{
+			Gtk.CellRendererText crt = renderer as Gtk.CellRendererText;
+			crt.Ellipsize = Pango.EllipsizeMode.End;
+			Notebooks.Notebook notebook = model.GetValue (iter, 0) as Notebooks.Notebook;
+			if (notebook == null) {
+				crt.Text = String.Empty;
+				return;
+			}
+			
+			crt.Text = notebook.Name;
 
-                   Tag tag_to_remove = null;
-                   foreach (Tag tag in selected_tags.Keys) {
-                    if (string.Compare (tag.NormalizedName, tag_name) == 0) {
-                     tag_to_remove = tag;
-                     break;
-                    }
-                   }
+			if (notebook is Notebooks.AllNotesNotebook) {
+				// Bold the All Notes item
+				crt.Markup = string.Format ("<span weight=\"bold\">{0}</span>", notebook.Name);
+			} else {
+				crt.Text = notebook.Name;
+			}
+		}
+		
+		private void OnNotebookSelectionChanged (object sender, EventArgs args)
+		{
+			Notebooks.Notebook notebook = GetSelectedNotebook ();
+			if (notebook == null) {
+				// Clear out the currently selected tags so that no notebook is selected
+				selected_tags.Clear ();
+                Tomboy.ActionManager ["OpenNotebookTemplateNoteAction"].Sensitive = false;
+				Tomboy.ActionManager ["DeleteNotebookAction"].Sensitive = false;
+				
+				// Select the "All Notes" item without causing
+				// this handler to be called again
+				notebooksTree.Selection.Changed -= OnNotebookSelectionChanged;
+				SelectAllNotesNotebook ();
+				notebooksTree.Selection.Changed += OnNotebookSelectionChanged;
+			} else {
+				selected_tags.Clear ();
+				if (notebook.Tag != null)
+					selected_tags.Add (notebook.Tag, notebook.Tag);
+				if (notebook is Notebooks.AllNotesNotebook) {
+	                Tomboy.ActionManager ["OpenNotebookTemplateNoteAction"].Sensitive = true;
+					Tomboy.ActionManager ["DeleteNotebookAction"].Sensitive = false;
+				} else {
+	                Tomboy.ActionManager ["OpenNotebookTemplateNoteAction"].Sensitive = true;
+					Tomboy.ActionManager ["DeleteNotebookAction"].Sensitive = true;
+				}
+			}
+			
+			UpdateResults ();
+		}
+		
+		void OnNewNotebook (object sender, EventArgs args)
+		{
+			Notebooks.NotebookManager.PromptCreateNewNotebook (this);
+		}
+		
+		void OnDeleteNotebook (object sender, EventArgs args)
+		{
+			Notebooks.Notebook notebook = GetSelectedNotebook ();
+			if (notebook == null)
+				return;
+			
+			Notebooks.NotebookManager.PromptDeleteNotebook (this, notebook);
+		}
 
-                   if (tag_to_remove != null) {
-                    selected_tags.Remove (tag_to_remove);
-                    UpdateResults ();
-                   }
-                  }
+		// Open the notebook's note template when activated
+		private void OnNotebookRowActivated (object sender, Gtk.RowActivatedArgs args)
+		{
+			OnOpenNotebookTemplateNote (sender, EventArgs.Empty);
+		}
+		
+		private void OnOpenNotebookTemplateNote (object sender, EventArgs args)
+		{
+			Notebooks.Notebook notebook = GetSelectedNotebook ();
+			if (notebook == null)
+				return;
+			
+			Note templateNote = notebook.GetTemplateNote ();
+			if (templateNote == null)
+				return; // something seriously went wrong
+			
+			templateNote.Window.Present ();
+		}
+		
+		private Notebooks.Notebook GetSelectedNotebook ()
+		{
+			Gtk.TreeModel model;
+			Gtk.TreeIter iter;
+			
+			if (notebooksTree.Selection.GetSelected (out model, out iter) == false)
+				return null; // Nothing selected
+			
+			return model.GetValue (iter, 0) as Notebooks.Notebook;
+		}
+		
+		private void SelectAllNotesNotebook ()
+		{
+			Gtk.TreeIter iter;
+			if (notebooksTree.Model.GetIterFirst (out iter) == true) {
+				notebooksTree.Selection.SelectIter (iter);
+			}
+		}
+		
+		[GLib.ConnectBefore]
+		void OnNotebooksTreeButtonPressed (object sender, Gtk.ButtonPressEventArgs args)
+		{
+			switch (args.Event.Button) {
+				case 3: // third mouse button (right-click)
+					Notebooks.Notebook notebook = GetSelectedNotebook ();
+					if (notebook == null)
+						return; // Don't pop open a submenu
+					
+					Gtk.TreePath path = null;
+					Gtk.TreeViewColumn column = null;
 
-                  void OnTagToggled (object sender, Gtk.ToggledArgs args)
-                  {
-			//Console.WriteLine("Tag was toggled!{0}");
-                   Gtk.CellRendererToggle crt = sender as Gtk.CellRendererToggle;
+					bool rowClicked = true;
 
-                   Gtk.TreePath path = new Gtk.TreePath (args.Path);
-                   Gtk.TreeIter iter;
-                   if (tags_store.GetIter (out iter, path) == false)
-                    return;
-			//Console.WriteLine("iter is true");
-                   Tag tag = tags_store.GetValue (iter, 0) as Tag;
-                   if (tag == null)
-                    return;
-			//Console.WriteLine("tag isn't null ");
-                   if (crt.Active) {
-                    // Uncheck the tag (remove the tag from selected_tags)
-                    if (selected_tags.ContainsKey (tag))
-                     selected_tags.Remove (tag);
-                   } else {
-                    // Check the tag (add it to selected_tags)
-                    selected_tags [tag] = tag;
-                   }
+					if (notebooksTree.GetPathAtPos ((int) args.Event.X,
+							(int) args.Event.Y, out path, out column) == false)
+						rowClicked = false;
 
-                   UpdateResults ();
-                  }
+					Gtk.TreeSelection selection = notebooksTree.Selection;
+					if (selection.CountSelectedRows () == 0)
+						rowClicked = false;
 
-                  void TagsToggleCellDataFunc (Gtk.TreeViewColumn tree_column,
-                    Gtk.CellRenderer cell, Gtk.TreeModel tree_model,
-                    Gtk.TreeIter iter)
-                  {
-                   Gtk.CellRendererToggle crt = cell as Gtk.CellRendererToggle;
-                   Tag tag = tree_model.GetValue (iter, 0) as Tag;
-                   if (tag == null)
-                    crt.Active = false;
-                   else {
-                    crt.Active = selected_tags.ContainsKey (tag);
-                   }
-                  }
+					Gtk.Menu menu = null;
+					if (rowClicked)
+						menu = Tomboy.ActionManager.GetWidget (
+							"/NotebooksTreeContextMenu") as Gtk.Menu;
+					else
+						menu = Tomboy.ActionManager.GetWidget (
+							"/NotebooksTreeNoRowContextMenu") as Gtk.Menu;
+					
+					PopupContextMenuAtLocation (menu,
+												(int) args.Event.X,
+												(int) args.Event.Y);
 
-                  void TagsNameCellDataFunc (Gtk.TreeViewColumn tree_column,
-                    Gtk.CellRenderer cell, Gtk.TreeModel tree_model,
-                    Gtk.TreeIter iter)
-                  {
-                   Gtk.CellRendererText crt = cell as Gtk.CellRendererText;
-                   Tag tag = tree_model.GetValue (iter, 0) as Tag;
-                   if (tag == null)
-                    crt.Text = String.Empty;
-                   else
-                    crt.Text = tag.Name;
-                  }
+				break;
+			}
+		}
+
+		void OnNotebooksKeyPressed (object sender, Gtk.KeyPressEventArgs args)
+		{
+			switch (args.Event.Key) {
+				case Gdk.Key.Escape:
+					// Allow Escape to close the window
+					OnCloseWindow (this, EventArgs.Empty);
+					break;
+				case Gdk.Key.Menu:
+					// Pop up the context menu if a notebook is selected
+					Notebooks.Notebook notebook = GetSelectedNotebook ();
+					if (notebook == null || notebook is Notebooks.AllNotesNotebook)
+						return; // Don't pop open a submenu
+					
+					Gtk.Menu menu = Tomboy.ActionManager.GetWidget (
+						"/NotebooksTreeContextMenu") as Gtk.Menu;
+					PopupContextMenuAtLocation (menu, 0, 0);
+
+					break;
+			}
+		}
+		
+		private void OnNoteAddedToNotebook (Note note, Notebooks.Notebook notebook)
+		{
+			UpdateResults ();
+		}
+		
+		private void OnNoteRemovedFromNotebook (Note note, Notebooks.Notebook notebook)
+		{
+			UpdateResults ();
+		}
 
                 public string SearchText
                 {
